# deps
from django_bulk_update.helper import bulk_update as ext_bulk_update


def bulk_update(objects, manager, **bulk_update_kwargs):
    """Updates the list of objects using django queryset's
    inbuilt ``.bulk_update()`` method if present, else
    django_bulk_update's ``bulk_update()`` will be used

    :param objects: list of objects that needs to be bulk updated
    :type objects: list[object]
    :param manager: instance of django model manager
    :type manager: models.Manager
    :param bulk_update_kwargs: keyword arguments passed to the ``bulk_update()``
    :return: None
    :rtype: None
    """
    try:
        manager.bulk_update(objects, **bulk_update_kwargs)
    except AttributeError:
        ext_bulk_update(objects, **bulk_update_kwargs)



# deps
from django.test import TestCase

# local
from anon.compat import bulk_update

from .compat import mock


class CompatTestCase(TestCase):
    @mock.patch("anon.compat.ext_bulk_update")
    def test_call_inbuilt_bulk_update(self, ext_bulk_update):
        class Manager:
            def bulk_update(self, objects, **kwargs):
                pass

        manager = Manager()

        class Obj(object):
            def __init__(self):
                self.first_name = "zzz"

        obj = Obj()

        self.assertTrue(hasattr(manager, "bulk_update"))
        bulk_update(objects=[obj], manager=manager)
        ext_bulk_update.assert_not_called()

    @mock.patch("anon.compat.ext_bulk_update")
    def test_call_ext_bulk_update(self, ext_bulk_update):
        class Manager:
            pass

        manager = Manager()

        class Obj(object):
            def __init__(self):
                self.first_name = "zzz"

        obj = Obj()
        self.assertFalse(hasattr(manager, "bulk_update"))
        bulk_update(
            objects=[obj], manager=manager, batch_size=42, update_fields=["first_name"]
        )
        ext_bulk_update.assert_called_once_with(
            [obj], batch_size=42, update_fields=["first_name"]
        )
